@@ -1,14 +1,14 @@
 /********************************************************************************
    zfuncs.h      include file for zfuncs functions
 
-   Copyright 2006-2017 Michael Cornelison
-   source code URL: http://kornelix.net
+   Copyright 2007-2018 Michael Cornelison
+   source code URL: https://kornelix.net
    contact: kornelix@posteo.de
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version. See http://www.gnu.org/licenses
+   (at your option) any later version. See https://www.gnu.org/licenses
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +17,7 @@
 
 *********************************************************************************/
 
-//     zfuncs.h   version  v.6.7
+//     zfuncs.h   version  v.6.8
 
 #include <sys/sysinfo.h>
 #include <sys/time.h>
@@ -40,6 +40,7 @@
 #include <signal.h>
 #include <execinfo.h>
 #include <locale.h>
+#include <glob.h>
 #include <gtk/gtk.h>
 
 #define VERTICAL GTK_ORIENTATION_VERTICAL                                        //  GTK shortcuts
@@ -206,7 +207,7 @@ char * formatKBMB(double fnum, int prec);
 int MatchWild(cchar * wildstr, cchar * str);                                     //  wildcard string match (match = 0)
 int MatchWildIgnoreCase(cchar * wildstr, cchar * str);                           //  wildcard string match, ignoring case
 cchar * SearchWild(cchar *wpath, int &flag);                                     //  wildcard file search
-cchar * SearchWildIgnoreCase(cchar *wpath, int &flag);                           //  wildcard file search, ignoring case
+cchar * SearchWildCase(cchar *wpath, int &flag);                                 //  wildcard file search, ignoring case
 
 //  search and sort functions ===================================================
 
@@ -281,14 +282,14 @@ int zinitapp(cchar *appname, cchar *homedir = 0);
 cchar * get_zprefix();                                                           //  get /usr or /usr/local  ... 
 cchar * get_zhomedir();                                                          //  get /home/user/.appname/
 cchar * get_zdatadir();                                                          //  get data directory
+cchar * get_zimagedir();                                                         //  get image directory
 cchar * get_zdocdir();                                                           //  get document directory
 cchar * get_zicondir();                                                          //  get icon directory
 
-void get_hardware_info(GtkWidget *widget);                                       //  get display, screen, mouse, gtksettings
 void zsetfont(cchar *newfont);                                                   //  set new app font and size
+int  widget_font_metrics(GtkWidget *widget, int &fww, int &fhh);                 //  get widget font char width/height
 
 int  locale_filespec(cchar *ftype, cchar *fname, char *filespec);                //  get a locale dependent file
-
 void showz_userguide(cchar *context = 0);                                        //  show user guide in new process
 void showz_logfile();                                                            //  show log file in popup window
 void showz_textfile(cchar *type, cchar *file);                                   //  show text file [.gz] in popup window
@@ -303,13 +304,13 @@ void ZTXinit(cchar *lang);
 cchar * ZTX(cchar *english);                                                     //  get translation for English message
 cchar * ZTX_missing(int &ftf);                                                   //  get missing translations, one per call
 
+
 /********************************************************************************
    GTK utility functions
 *********************************************************************************/
 
 void zmainloop(int skip = 0);                                                    //  do main loop, process menu events
 void zmainsleep(float secs);                                                     //  do main loop and sleep designated time
-void zthreadcrash();                                                             //  crash if thread is not main() thread
 
 //  cairo drawing region for GDK window      GTK 3.21 version
 
@@ -334,28 +335,32 @@ typedef struct {
 cairo_t * draw_context_create(GdkWindow *gdkwin, draw_context_t &context);
 void draw_context_destroy(draw_context_t &context);
 
-//  text window print and read utilities
 
-void wprintx(GtkWidget *Win, int line, cchar *mess, int bold = 0);               //  write text to line, optional bold
-void wprintf(GtkWidget *Win, int line, cchar *format, ...);                      //  "printf" version
-void wprintf(GtkWidget *Win, cchar *format, ... );                               //  "printf" to next line, scroll up
-void wprintxx(GtkWidget *Win, int line, cchar *mess, int bold = 0);              //  alternate with auto zmainloop()
-void wprintff(GtkWidget *Win, int line, cchar *format, ...);                     //  alternate with auto zmainloop()
-void wprintff(GtkWidget *Win, cchar *format, ... );                              //  alternate with auto zmainloop()
-void wscroll(GtkWidget *mLog, int line);                                         //  scroll window to put line on screen
-void wclear(GtkWidget *Win);                                                     //  clear window
-void wclear(GtkWidget *Win, int line);                                           //  clear from line to end
-char * wscanf(GtkWidget *Win, int &ftf);                                         //  get text lines from edit widget
-int  wfiledump(GtkWidget *Win, char *filespec);                                  //  dump text window to file
-void wfilesave(GtkWidget *Win, GtkWindow *parent);                               //  wfiledump() via file-chooser dialog
-void wprintp(GtkWidget *Win);                                                    //  print text window to default printer
+/********************************************************************************/
 
-//  intercept text window mouse click functions
+//  textwidget functions - scrollable text widget for text reports and line editing
+//  widget = zdialog_widget(zd,textwidget) where textwidget is a zdialog "text" widget type
 
-typedef void clickfunc_t(GtkWidget *widget, int line, int pos);                  //  function to get clicked text position
-void textwidget_set_clickfunc(GtkWidget *widget, clickfunc_t clickfunc);         //  (wrapped lines are one logical line)
-char * textwidget_get_line(GtkWidget *widget, int line, int hilite);             //  get entire line at clicked position
-char * textwidget_get_word(char *line, int pos, cchar *dlims, char &end);        //  get delimited word at clicked position
+void textwidget_clear(GtkWidget *widget);                                              //  clear all text
+void textwidget_clear(GtkWidget *widget, int line);                                    //  clear text from line to end
+int  textwidget_linecount(GtkWidget *widget);                                          //  get current line count
+void textwidget_append(GtkWidget *widget, int bold, cchar *format, ...);               //  append line
+void textwidget_insert(GtkWidget *widget, int bold, int line, cchar *format, ...);     //  insert line 
+void textwidget_replace(GtkWidget *widget, int bold, int line, cchar *format, ...);    //  replace line
+void textwidget_delete(GtkWidget *widget, int line);                                   //  delete line
+int  textwidget_find(GtkWidget *widget, char *matchtext, int line1);                   //  find matching line
+void textwidget_scroll(GtkWidget *widget, int line);                                   //  scroll to make line visible
+void textwidget_dump(GtkWidget *widget, cchar *filename);                              //  dump all text into a file
+void textwidget_save(GtkWidget *widget, GtkWindow *parent);                            //  same, with save-as dialog
+char * textwidget_line(GtkWidget *widget, int line, int strip);                        //  retrieve line (strip \n) 
+void textwidget_highlight_line(GtkWidget *widget, int line);                           //  highlight line
+char * textwidget_word(GtkWidget *, int line, int posn, cchar *dlims, char &end);      //  retrieve word
+void textwidget_highlight_word(GtkWidget *widget, int line, int posn, int cc);         //  highlight word
+typedef void textwidget_callbackfunc_t(GtkWidget *, int line, int posn, int KBkey);    //  callback function to receive
+void textwidget_set_callbackfunc(GtkWidget *, textwidget_callbackfunc_t func);         //    clicked line/posn or KB press
+
+
+/********************************************************************************/
 
 //  functions to simplify building menus, tool bars, status bars
 
@@ -378,6 +383,7 @@ GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *lab, cchar *tip,
 GtkWidget * create_stbar(GtkWidget *vbox);                                       //  create status bar in packing box
 int stbar_message(GtkWidget *stbar, cchar *message);                             //  display message in status bar
 
+
 /********************************************************************************/
 
 GtkWidget * create_popmenu();                                                    //  create an empty popup menu
@@ -385,13 +391,15 @@ GtkWidget * add_popmenu_item(GtkWidget *popmenu, cchar *mname,
                            cbFunc func, cchar *arg, cchar *mtip = 0);
 void popup_menu(GtkWidget *, GtkWidget *popmenu);                                //  pop-up menu at current mouse posn.
 
+
 /********************************************************************************/
 
 //  user editable graphic menu in popup window
 //  menus can be added and arranged using the mouse
 
-typedef void gmenuz_cbfunc(cchar *menu);                                         //  caller-supplied callback function
-void gmenuz(GtkWidget *parent, cchar *title, cchar *ufile, gmenuz_cbfunc);       //  show window, handle mouse drag/click
+typedef void Gmenuz_cbfunc(cchar *menu);                                         //  caller-supplied callback function
+void Gmenuz(GtkWidget *parent, cchar *title, cchar *ufile, Gmenuz_cbfunc);       //  show window, handle mouse drag/click
+
 
 /********************************************************************************/
 
@@ -415,15 +423,19 @@ struct Vmenu {
    GtkWidget   *vbox;                                                            //  parent window (container)
    GtkWidget   *topwin;                                                          //  top-level window of parent
    GtkWidget   *layout;                                                          //  drawing window
+   float       fgRGB[3];                                                         //  font color, RGB scaled 0-1
+   float       bgRGB[3];                                                         //  background color, RGB scaled 0-1
    int         xmax, ymax;                                                       //  layout size
    int         mcount;                                                           //  menu entry count
    vmenuent    menu[100];
 };
 
-Vmenu *Vmenu_new(GtkWidget *vbox);                                               //  create new menu in parent vbox
+Vmenu *Vmenu_new(GtkWidget *vbox, float fgRGB[3], float bgRGB[3]);               //  create new menu in parent vbox
 void Vmenu_add(Vmenu *vbm, cchar *name, cchar *icon,                             //  add menu item with response function
                int iconww, int iconhh, cchar *desc,
                cbFunc func, cchar *arg);                                         //  function may be popup_menu()
+void Vmenu_block(int flag);                                                      //  block or unblock menu
+
 
 /********************************************************************************/
 
@@ -456,13 +468,15 @@ struct zwidget  {
 
 struct zdialog  {
       int         sentinel1;                       //  validity sentinel1
-      int         uniqueID;                        //  unique ID, monotone increasing              
-      void        *eventCB;                        //  widget event user callback function
+      int         uniqueID;                        //  unique ID, monotone increasing
+      char        *title;                          //  dialog title
+      void        *eventCB;                        //  dialog event user callback function
       int         zrunning;                        //  dialog is running (0,1)
       int         zstat;                           //  dialog status (from completion button)
       int         disabled;                        //  widget signals/events are disabled
       int         saveposn;                        //  save and recall window position each use
       int         saveinputs;                      //  save and recall user inputs each use
+      GtkWidget   *dialog;                         //  dialog window
       GtkWidget   *parent;                         //  parent window or null
       cchar       *compbutton[zdmaxbutts];         //  dialog completion button labels
       GtkWidget   *compwidget[zdmaxbutts];         //  dialog completion button widgets
@@ -538,9 +552,19 @@ int zdialog_inputs(cchar *action);
 int zdialog_save_inputs(zdialog *zd);                                            //  save zdialog input fields when finished
 int zdialog_restore_inputs(zdialog *zd);                                         //  restore zdialog inputs from prior use
 
-//  write text to popup window, shell command to popup window
+void window_to_mouse(GtkWidget *window);                                         //  move GtkWidget/window to mouse position
+
+//  write text report in popup window
+
+GtkWidget * popup_report_open(GtkWidget *paren, cchar *titl, int ww, int hh);    //  open popup text report window
+void popup_report_header(int bold, cchar *format, ...);                          //  write non-scrolling header line
+void popup_report_write(int bold, cchar *format, ...);                           //  write text line
+void popup_report_top();                                                         //  go to top of window
+void popup_report_move(int ww, int hh);                                          //  move window ww/hh pixels
+void popup_report_close(int secs);                                               //  close window in given seconds
+
+//  shell command to popup window
 
-GtkWidget * write_popup_text(cchar *action, cchar *text = 0, int ww = 0, int hh = 0, GtkWidget *parent = 0);
 int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent = 0, int top = 0);
 
 //  popup message dialogs
@@ -548,6 +572,7 @@ int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent
 void zmessageACK(GtkWidget *parent, cchar *format, ... );                                 //  display message, wait for OK
 int  zmessageYN(GtkWidget *parent, cchar *format, ... );                                  //  display message, wait for YES/NO
 zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *format, ...);                 //  show message, timeout or cancel
+zdialog * zmessage_post_bold(GtkWidget *parent, int secs, cchar *format, ...);            //   " " with big red bold font
 char * zdialog_text(GtkWidget *parent, cchar *title, cchar *inittext);                    //  get short text input from user
 int  zdialog_choose(GtkWidget *parent, cchar *message, ...);                              //  show message, return choice
 void poptext_mouse(GtkWidget *parent, cchar *text, int dx, int dy, float s1, float s2);   //  show popup text at mouse
@@ -581,6 +606,7 @@ PIXBUF * text_pixbuf(cchar *text, cchar *font, int fsize, GtkWidget *);
 
 int  move_pointer(GtkWidget *, int px, int py);                                  //  move the mouse pointer to px, py
 
+
 /********************************************************************************
    C++ classes
 *********************************************************************************/
